@@ -1,7 +1,7 @@
 # coding: utf-8
 """
-    :mod:`sqlamp` --- Materialized Path for SQLAlchemy
-    ==================================================
+    `sqlamp` --- Materialized Path for SQLAlchemy
+    =============================================
 
     :author: `Anton Gritsay <anton@angri.ru>`_
     :version: %(version)s
@@ -14,7 +14,7 @@
 
     `Materialized Path` is a way to store (and fetch) a trees in a relational
     databases. It is the compromise between `Nested Sets` and `Adjacency
-    Relations` in respect to simplicity and efficiency. Method was proposed
+    Relations` in respect to simplicity and efficiency. Method was promoted
     by `Vadim Tropashko`_ in his book `SQL Design Patterns`_. Vadim's
     description of the method can be read in his article `Trees in SQL:
     Nested Sets and Materialized Path (by Vadim Tropashko)`_.
@@ -29,11 +29,13 @@
         * Fetching node's descendants, ancestors and children using the most
           efficient way available (see :class:`MPInstanceManager`)
         * Autochecking exhaustion of tree size limits --- maximum number of
-          children, maximum nesting level (see :class:`MPManager` to learn
+          children and maximum nesting level (see :class:`MPManager` to learn
           more about limits fine-tuning) is done during session flush.
         * Rebuilding all trees (see :meth:`MPClassManager.rebuild_all_trees`)
           and any subtree (:meth:`MPClassManager.rebuild_subtree`) on the
           basis of Adjacency Relations.
+        * Collapsing flat tree returned from query to recursive structure (see
+          :func:`tree_recursive_iterator`).
 
     Moving of nodes is not yet implemented.
 
@@ -52,15 +54,16 @@
     .. _`PostgreSQL`: http://postgresql.org
 """
 import weakref
+from operator import attrgetter
 import sqlalchemy, sqlalchemy.orm
 
 
 __all__ = [
-    'MPManager',
+    'MPManager', 'tree_recursive_iterator',
     'PathOverflowError', 'TooManyChildrenError', 'PathTooDeepError'
 ]
 
-__version__ = (0, 3)
+__version__ = (0, 4)
 __doc__ %= {'version': '.'.join(map(str, __version__))}
 
 
@@ -146,7 +149,7 @@ class MPOptions(object):
         self.steplen = steplen
 
         self.max_children = len(ALPHABET) ** steplen
-        self.max_depth = PATH_FIELD_LENGTH / steplen
+        self.max_depth = (PATH_FIELD_LENGTH / steplen) + 1
 
         self.path_field = sqlalchemy.Column(
             path_field_name, PathField(PATH_FIELD_LENGTH), nullable=False
@@ -203,14 +206,27 @@ class _InsertionsParamsSelector(object):
         and `depth` fields and put them in dict `self._query_result`.
         """
         opts = self._mp_opts
-        if self.parent_id is not None:
+        if self.parent_id is None:
+            # a new instance will be a root node.
+            # `tree_id` is next unused integer value,
+            # `depth` for root nodes is equal to zero,
+            # `path` should be empty string.
+            query = self.session.execute(sqlalchemy.select([
+                (sqlalchemy.func.coalesce(
+                    sqlalchemy.func.max(opts.tree_id_field), 0)
+                 + 1).label('tree_id')
+            ])).fetchone()
+            self._query_result = dict(
+                path='', depth=0, tree_id=query['tree_id']
+            )
+        else:
             # a new instance has at least one ancestor.
             # `tree_id` can be used from parent's value,
             # `depth` is parent's depth plus one,
             # `path` will be calculated from two values -
             # the path of the parent node itself and it's
             # last child's path.
-            query = sqlalchemy.select(
+            query = self.session.execute(sqlalchemy.select(
                 [
                     opts.tree_id_field.label('tree_id'),
                     (opts.depth_field + 1).label('depth'),
@@ -221,46 +237,23 @@ class _InsertionsParamsSelector(object):
                     ).label('last_child_path'),
                 ],
                 opts.pk_field == self.parent_id
+            )).fetchone()
+            steplen = self._mp_opts.steplen
+            if not query['last_child_path']:
+                # node is the first child.
+                path = query['parent_path'] + ALPHABET[0] * steplen
+            else:
+                try:
+                    path = inc_path(query['last_child_path'], steplen)
+                except PathOverflowError:
+                    # transform exception `PathOverflowError`, raised by
+                    # `inc_path()` to more convinient `TooManyChildrenError`.
+                    raise TooManyChildrenError()
+            if len(path) > PATH_FIELD_LENGTH:
+                raise PathTooDeepError()
+            self._query_result = dict(
+                path=path, depth=query['depth'], tree_id=query['tree_id']
             )
-        else:
-            # a new instance will be a root node.
-            # `tree_id` is next unused integer value,
-            # `depth` for root nodes is equal to zero,
-            # `path` should be `ALPHABET[0] * opts.steplen`.
-            query = sqlalchemy.select(
-                [
-                    (sqlalchemy.func.coalesce(
-                        sqlalchemy.func.max(opts.tree_id_field), 0
-                     ) + 1).label('tree_id'),
-                    sqlalchemy.sql.literal(0).label('depth'),
-                    sqlalchemy.sql.literal('').label('parent_path'),
-                    sqlalchemy.sql.literal(None).label('last_child_path')
-                ]
-            )
-        result = self.session.execute(query).fetchone()
-        # `tree_id` and `depth` values can be used
-        # directly from the query we made:
-        self._query_result = dict(
-            tree_id=result['tree_id'], depth=result['depth']
-        )
-
-        parent_path = result['parent_path']
-        last_child_path = result['last_child_path']
-        steplen = self._mp_opts.steplen
-        if not last_child_path:
-            # node is root or first child.
-            path = parent_path + ALPHABET[0] * steplen
-        else:
-            try:
-                path = inc_path(last_child_path, steplen)
-            except PathOverflowError:
-                # transform exception `PathOverflowError`, raised by
-                # `inc_path()` to more convinient `TooManyChildrenError`.
-                raise TooManyChildrenError()
-
-        if len(path) > PATH_FIELD_LENGTH:
-            raise PathTooDeepError()
-        self._query_result['path'] = path
 
     @property
     def query_result(self):
@@ -345,10 +338,12 @@ class MPClassManager(object):
     Node class manager. No need to create it by hand: it created
     by :class:`MPManager`.
 
+    :param node_class: class which was mapped to tree table.
     :param opts: instance of :class:`MPOptions`
     """
-    def __init__(self, opts):
+    def __init__(self, node_class, opts):
         self._mp_opts = opts
+        self.node_class = node_class
         self.mapper_extension = MPMapperExtension(opts)
 
     @property
@@ -445,6 +440,10 @@ class MPClassManager(object):
         order_by = order_by or opts.pk_field
         for index in opts.indices:
             index.drop()
+        # resetting the path for roots:
+        opts.table.update().where(opts.parent_id_field == None) \
+                           .values({opts.path_field: ''}) \
+                           .execute()
         roots = sqlalchemy.select(
             [opts.pk_field], opts.parent_id_field == None
         ).order_by(order_by)
@@ -454,6 +453,19 @@ class MPClassManager(object):
         for index in opts.indices:
             index.create()
 
+    def query_all_trees(self, session):
+        """
+        Query all stored trees.
+
+        :param session: a sqlalchemy `Session` object to bind a query.
+        :returns:
+            `Query` object whith all nodes of all trees sorted as usual
+            by `(tree_id, path)`.
+        """
+        query = sqlalchemy.orm.Query(self.node_class, session=session) \
+                              .order_by(self)
+        return query
+
 
 class MPInstanceManager(object):
     """
@@ -699,7 +711,7 @@ class MPManager(object):
     :param steplen=3:
         integer, the number of characters in each part of the path.
         This value allows to fine-tune the limits for max tree depth
-        (equal to `255 / steplen`) and max children in each node
+        (equal to `(255 / steplen) + 1`) and max children in each node
         (`36 ** steplen`). Default `3` value sets the following limits:
         max depth is `85` and max children number is `46656`.
     :param instance_manager_key='_mp_instance_manager':
@@ -715,7 +727,7 @@ class MPManager(object):
                                                '_mp_instance_manager')
         opts = MPOptions(*args, **kwargs)
         self._mp_opts = opts
-        self.class_manager = MPClassManager(opts)
+        self.class_manager = None
 
     def __get__(self, obj, objtype):
         """
@@ -723,6 +735,8 @@ class MPManager(object):
         or :class:`MPClassManager` if via class.
         """
         if obj is None:
+            if self.class_manager is None:
+                self.class_manager = MPClassManager(objtype, self._mp_opts)
             return self.class_manager
         else:
             instance_manager = obj.__dict__.get(self.instance_manager_key)
@@ -731,3 +745,118 @@ class MPManager(object):
                 obj.__dict__[self.instance_manager_key] = instance_manager
             return instance_manager
 
+
+_nonexistent = object()
+def _iter_current_next(sequence):
+    """
+    Generate `(current, next)` tuples from sequence. Last tuple will
+    have `_nonexistent` object at the second place.
+
+    >>> x = _iter_current_next('1234')
+    >>> x.next(), x.next(), x.next()
+    (('1', '2'), ('2', '3'), ('3', '4'))
+    >>> x.next() == ('4', _nonexistent)
+    True
+    >>> list(_iter_current_next(''))
+    []
+    >>> list(_iter_current_next('1')) == [('1', _nonexistent)]
+    True
+    """
+    iterator = iter(sequence)
+    current_item = iterator.next()
+    while current_item != _nonexistent:
+        next_item = next(iterator, _nonexistent)
+        yield (current_item, next_item)
+        current_item = next_item
+
+def _recursive_iterator(sequence, is_child_func):
+    """
+    Make a recursive iterator from plain sequence using :attr:`is_child_func`
+    to determine parent-children relations. Works right only if used in
+    depth-first recursive consumer.
+
+    :param is_child_func:
+        a callable object which accepts two positional arguments and
+        returns `True` value if first argument value is parent of second
+        argument value.
+
+    >>> is_child_func = lambda parent, child: child > parent
+    >>> def listify(seq):
+    ...     return [(node, listify(children)) for node, children in seq]
+    >>> listify(_recursive_iterator('ABCABB', is_child_func))
+    [('A', [('B', [('C', [])])]), ('A', [('B', []), ('B', [])])]
+    >>> listify(_recursive_iterator('', is_child_func))
+    []
+    >>> _recursive_iterator('A', is_child_func).next()
+    ('A', ())
+    >>> _recursive_iterator('AB', is_child_func).next() # doctest: +ELLIPSIS
+    ('A', <generator object ...>)
+    """
+    current_next_iterator = _iter_current_next(sequence)
+    item = {}
+    is_parent_of_next = lambda node: \
+            item['next'] is not _nonexistent \
+            and is_child_func(node, item['next'])
+
+    def step():
+        item['current'], item['next'] = current_next_iterator.next()
+        if is_parent_of_next(item['current']):
+            return (item['current'], children_generator(item['current']))
+        else:
+            return (item['current'], tuple())
+
+    def children_generator(parent_node):
+        while True:
+            yield step()
+            if not is_parent_of_next(parent_node):
+                break
+
+    while True:
+        yield step()
+
+
+def tree_recursive_iterator(flat_tree, class_manager):
+    """
+    Make a recursive iterator from plain tree nodes sequence (`Query`
+    instance for example). Generates two-item tuples: node itself
+    and it's children collection (which also generates two-item tuples...)
+    Children collection evaluates to ``False`` if node has no children
+    (it is zero-length tuple for leaf nodes), else it is a generator object.
+
+    :param flat_tree: plain sequence of tree nodes.
+    :param class_manager: instance of :class:`MPClassManager`
+
+    Can be used when it is simpler to process tree structure recursively.
+    Simple usage example::
+
+        def recursive_tree_processor(nodes):
+            print '<ul>'
+            for node, children in nodes:
+                print '<li>%s' % node.name,
+                if children:
+                    recursive_tree_processor(children)
+                print '</li>'
+            print '</ul>'
+
+        query = root_node.mp.query_descendants(and_self=True)
+        recursive_tree_processor(
+            sqlamp.tree_recursive_iterator(query, Node.mp)
+        )
+
+    If `flat_tree` is a `sqlalchemy.orm.Query` instance, it will be ordered
+    by `class_manager`. If it is plain list, do not forget that such ordering
+    is strictly required for `tree_recursive_iterator()` to work right.
+
+    .. warning:: Process `flat_tree` items once and sequentially so works
+      right only if used in depth-first recursive consumer.
+    """
+    opts = class_manager._mp_opts
+    tree_id = attrgetter(opts.tree_id_field.name)
+    depth = attrgetter(opts.depth_field.name)
+    def is_child(parent, child):
+        return tree_id(parent) == tree_id(child) \
+                and depth(child) == depth(parent) + 1
+    if isinstance(flat_tree, sqlalchemy.orm.Query):
+        flat_tree = flat_tree.order_by(class_manager)
+    return _recursive_iterator(flat_tree, is_child)
+
